#pragma once
#include "Vec2.h"
#include <stdlib.h>
#include <list>

typedef std::list<Vec2> VertexList;

struct AABB {
	Vec2 p1;
	Vec2 p2;
};

class Collider
{
public:
	Vec2 WorldMin() { return center + aabb.p1; }
	Vec2 WorldMax() { return center + aabb.p2; }
	virtual void CollidedWith(Collider *other) {};
	Vec2 velocity;
	bool isStatic;
	bool isTrigger;
	AABB aabb;
	Vec2 center;
protected:

};
typedef std::list<Collider *> Colliders;

// Let's use relative verts.
class PolyCollider : public Collider
{
public:
	PolyCollider(int numVerts, Vec2 center, Vec2 nverts[], bool isStatic=false, bool isTrigger=false) {
		this->numVerts = numVerts;
		this->center = center;
		verts = new Vec2[numVerts];
		edges = new Vec2[numVerts];
		aabb.p1 = Vec2(999999, 999999);
		aabb.p2 = Vec2(-999999, -999999);

		for(int i = 0; i < numVerts; i++) {
			verts[i] = center + nverts[i];
			edges[i] = nverts[(i+1) % numVerts] - nverts[i];
			if(nverts[i].x < aabb.p1.x) aabb.p1.x = nverts[i].x;
			if(nverts[i].x > aabb.p2.x) aabb.p2.x = nverts[i].x;
			if(nverts[i].y < aabb.p1.y) aabb.p1.y = nverts[i].y;
			if(nverts[i].y > aabb.p2.y) aabb.p2.y = nverts[i].y;
		}
		this->isStatic = isStatic;
		this->isTrigger = isTrigger;
	}

	PolyCollider() {
		verts = 0;
		edges = 0;
		isTrigger = false;
		isStatic = false;
	}
	
	~PolyCollider() {
		if(verts) delete [] verts;
		if(edges) delete [] edges;
	}

	void UpdateAABB() {	
		for(int i = 0; i < numVerts; i++) {
			if(verts[i].x < aabb.p1.x) aabb.p1.x = verts[i].x;
			if(verts[i].x > aabb.p2.x) aabb.p2.x = verts[i].x;
			if(verts[i].y < aabb.p1.y) aabb.p1.y = verts[i].y;
			if(verts[i].y > aabb.p2.y) aabb.p2.y = verts[i].y;
		}
	}

	Vec2 *verts;
	Vec2 *edges;
	int numVerts;
	void Update();
	void MoveTo(Vec2 pos);
	Vec2 GetCentroid();

private:
	

};

struct Collision {
	bool collides;
	PolyCollider *c1;
	PolyCollider *c2;
	bool willCollide;
	Vec2 translationVector;
};



typedef std::list<Collision> Collisions;


class CollisionManager {
public:
	CollisionManager() {collisionchecks = 0;};
	void UpdateCollision();
	void AddCollider(Collider *col);
	void DebugDraw();

private:

	Colliders colliders;
	Collisions collisions;
	
	void BroadPhase();
	void SAPSortColliders();
	void SAPBroadPhase(Collider *col);
	void BruteForceBroadPhase(Collider *col);
	Collision CheckCollision(PolyCollider *c1, PolyCollider *c2);
	float IntervalDistance(Vec2 minMax1, Vec2 minMax2);
	void DoNarrowPhase();
	int collisionchecks;

	// Helper Functions
	void ProjectPolygon(Vec2 axis, PolyCollider *polygon, Vec2 *minMax);
};

// Box collider def
	/*PolyCollider(Vec2 p1, Vec2 p2)
	{
		this->p1 = p1;
		this->p2 = p2;
		if(p2.x < p1.x)
		{	
			this->p1.x = p2.x;
			this->p2.x = p1.x;
		}
		if(p2.y < p1.y)
		{
			this->p1.y = p2.y;
			this->p2.y = p1.y;
		}
	}
	*/