#pragma once
#include <string>
#include <list>
#include "Vec2.h"
#include "SDL.h"
#include <FTGL/ftgl.h>

#define SC_WIDTH 1280
#define SC_HEIGHT 1024


using namespace std;

class Core;

enum GFXmode {
	GFXMODE_INVALID		= -1,
	GFXMODE_FULLSCREEN	= 0,
	GFXMODE_WINDOW		= 2,
};

// A generic color struct. Used for storing / adding colors.
struct Color {
	float r;
	float g;
	float b;
	float a;

	Color() {
		r = 1;
		g = 1;
		b = 0;
	}

	Color(float nr, float ng, float nb) {
		r = nr;
		g = ng;
		b = nb;
		a = 1.0f;
	};

	Color(float nr, float ng, float nb, float na) {
		r = nr;
		g = ng;
		b = nb;
		a = na;
	}

	Color operator+( Color &other ){
		r += other.r;
		g += other.g;
		b += other.b;
		a += other.a;
		if(r < 0)	r = 0;
		if(r > 1) r = 1;
		if(g < 0)	g = 0;
		if(g > 1) g = 1;
		if(b < 0)	b = 0;
		if(b > 1) b = 1;
		if(a < 0) a = 0;
		if(a > 1) a = 1;

		return Color(r,g,b,a);
	};
	Color operator-( Color &other ){
		return Color(r,g,b,a) + Color(-other.r, -other.g, -other.b, -other.a);
	};	

};

// Our graphics wrapper. All draw calls go through this class.
class GFX {
public:
	SDL_Surface *screen;
	static GFX* Instance();
	void GFXtriangle(float x1, float y1, float x2, float y2, float x3, float y3, Color col);
	void GFXrectfill(float x1, float y1, float x2, float y2, Color col);
	void GFXline(float x1, float y1, float x2, float y2, Color col);
	void GFXputpixel(float x, float y, Color col);
	void GFXclearTo(Color col);
	void GFXupdate();
	void GFXtext(char *str, float x, float y, Color col, Color mask);
	void GFXtext_center(char *str, float x, float y, Color col, Color mask);
	void GFXDebug(char *stre, double val, float x, float y);
	void GFXDebug(char *stre, Vec2 val);
	//BITMAP *GFXbuffer;
	~GFX();
	bool init();
	int drawcalls;
	int culled;

protected:
	GFX();
	GFX(GFXmode gfxmode, int screenh, int screenv);
private:
	// Will be used for resolution stuff in the future
	int SCREENH;
	int SCREENV;
	static GFX* instance;
	FTFont *debugFont;
	list<std::string> debugDisplay;
	Core *c;
};
