#include "Core.h"

// Iterate through the deletion queue, find the corresponding Renderer, remove it from the objects list.
// next, call the corresponding delete function based on the result of GetType(). Atm particle or entity.
void RenderManager::CleanUp() {
	for(Renderers::iterator ri = deletionQueue.begin(); ri != deletionQueue.end(); ri++) {
		Renderer *toDelete = (*ri);

		int i = 0;
		if(toDelete) {
			for(Renderers::iterator reni = objects.begin(); reni != objects.end(); reni++) {
				Renderer *curr = (*reni);

				if(curr == toDelete) {
					objects.erase(reni);

					// Does the object require any specific internal cleanup?
					curr->Destroy();
					break;
				}
			}
			// Actually free the memory.
			delete(toDelete);
			toDelete = 0;
		}
	}
	deletionQueue.clear();
}

void RenderManager::Update(float dt) {
	for(Renderers::iterator ri = objects.begin(); ri != objects.end(); ri++) {
		Renderer *ren = (*ri);
		ren->Update(dt);
	}

	CleanUp();
}

void RenderManager::Render() {
	for(Renderers::iterator ri = objects.begin(); ri != objects.end(); ri++) {
		(*ri)->Render();
	}
}

// Add an renderer to the object list.
void RenderManager::AddRenderer(Renderer *renderer) {
	objects.push_back(renderer);
}

// WARNING: This integrity check has been made optional in the interest of performance. If it becomes a problem, 
// it can be reinstated. All we need to do is any renderer that can be deleted from more than one place needs to have
// any deletes after the first called as safe (or all of them).
// ie if the same object is added to the deletionQueue more than once it'll try to delete the memory twice and
// cause issues.
void RenderManager::RemoveRenderer(Renderer *obj, bool safe) {
	if(safe) {
		for(Renderers::iterator ri = deletionQueue.begin(); ri != deletionQueue.end(); ri++) {
			Renderer *curr = (*ri);
			if(curr == obj)
				return;
		}
	}
	deletionQueue.push_back(obj);
}